#############################
#
# Revolving system
#
# Compute the arguments used to make a tilted circle
# Assume the trig numbers are already known
#
# rx and ry: angles of the normal vector of the circle
# radius: radius of the circle
# dt: Increment of t
#
#
# P(t) = r * cos(2*pi * t) * A + r * sin(2*pi * t) * B
#
#	  ┌  sin(ry) * sin(rx) ┐
# A = |     -cos(rx)       |
#     └ -cos(ry) * sin(rx) ┘
#
#
#	  ┌ -cos(ry) ┐
# B = |    0     |
#     └ -sin(ry) ┘
#
#
# P'(t) = r * (-sin(2*pi * t)) * 2*pi * A + r * cos(2*pi * t) * 2*pi * B
#
# => V = P'(t) * dt
#
#
# Made by Arth2000
#
#############################


# Precompute (-pi/2 * A) and (pi/2 * B)

# A_x = -sin(ry) * sin(rx) * 2*pi
scoreboard players operation @s A_x = @s sin_ry
scoreboard players operation @s A_x *= @s sin_rx
scoreboard players operation @s A_x /= -U CONST
scoreboard players operation @s A_x *= 2*pi CONST
scoreboard players operation @s A_x /= U CONST

# A_y = -cos(rx) * 2*pi
scoreboard players operation @s A_y = @s cos_rx
scoreboard players operation @s A_y *= 2*pi CONST
scoreboard players operation @s A_y /= U CONST

# A_z = cos(ry) * sin(rx) * pi/2
scoreboard players operation @s A_z = @s cos_ry
scoreboard players operation @s A_z *= @s sin_rx
scoreboard players operation @s A_z /= U CONST
scoreboard players operation @s A_z *= 2*pi CONST
scoreboard players operation @s A_z /= U CONST


# B_x = cos(ry) * 2*pi
scoreboard players operation @s B_x = @s cos_ry
scoreboard players operation @s B_x *= -1 CONST
scoreboard players operation @s B_x *= 2*pi CONST
scoreboard players operation @s B_x /= U CONST

# B_z = sin(ry) * 2*pi
scoreboard players operation @s B_z = @s sin_ry
scoreboard players operation @s B_z *= -1 CONST
scoreboard players operation @s B_z *= 2*pi CONST
scoreboard players operation @s B_z /= U CONST


# Set first t
scoreboard players operation @s t = @s dt
scoreboard players operation @s t /= -2 CONST


# Default values
scoreboard players set @s err_vx 0
scoreboard players set @s err_vy 0
scoreboard players set @s err_vz 0
scoreboard players set @s err_vx_U^2 0
scoreboard players set @s err_vy_U^2 0
scoreboard players set @s err_vz_U^2 0

scoreboard players set @s fx 0
scoreboard players set @s fy 0
scoreboard players set @s fz 0

scoreboard players set @s vx 0
scoreboard players set @s vy 0
scoreboard players set @s vz 0
