#############################
#
# Revolving system
#
# Handles the revolving entities
#
#
#
# P(t) = r * cos(2*pi * t) * A + r * sin(2*pi * t) * B
#
#	  ┌  sin(ry) * sin(rx) ┐
# A = |      cos(rx)       |
#     └ -cos(ry) * sin(rx) ┘
#
#
#	  ┌ -cos(ry) ┐
# B = |    0     |
#     └ -sin(ry) ┘
#
#
# P'(t) = r * -sin(2*pi * t) * 2*pi * A + r * cos(2*pi * t) * 2*pi * B
#
# => V = P'(t) * dt
#
#
# A and B have already been precomputed (see revolving:init)
#
#
# Made by Arth2000
#
#############################

# Update t
scoreboard players operation @s t += @s dt
scoreboard players operation @s t %= U CONST


# Computes P'(t)
# A part:
# sin(2*pi * t) * (-2*pi * A)
scoreboard players operation @s in1 = @s t
scoreboard players operation @s in1 *= 4 CONST
function utils:math/trig/sin5

scoreboard players operation @s fx = @s out1
scoreboard players operation @s fx *= @s A_x

scoreboard players operation @s fy = @s out1
scoreboard players operation @s fy *= @s A_y

scoreboard players operation @s fz = @s out1
scoreboard players operation @s fz *= @s A_z 


# B part:
# cos(2*pi * t) * (2*pi * B)
scoreboard players operation @s in1 = @s t
scoreboard players operation @s in1 *= 4 CONST
function utils:math/trig/cos5

scoreboard players operation @s tmp1 = @s out1
scoreboard players operation @s tmp1 *= @s B_x
scoreboard players operation @s fx += @s tmp1

scoreboard players operation @s tmp1 = @s out1
scoreboard players operation @s tmp1 *= @s B_z
scoreboard players operation @s fz += @s tmp1


# Scale down
scoreboard players operation @s fx += @s err_vx_U^3
scoreboard players operation @s err_vx_U^3 = @s fx
scoreboard players operation @s err_vx_U^3 %= U CONST
scoreboard players operation @s fx /= U CONST
scoreboard players operation @s fy /= U CONST
scoreboard players operation @s fz /= U CONST

# Multiply by dt
# Keeps track of the error in 1/U^2 in case dt is small and there is a huge imprecision
scoreboard players operation @s fx *= @s dt
scoreboard players operation @s fx += @s err_vx_U^2
scoreboard players operation @s err_vx_U^2 = @s fx
scoreboard players operation @s err_vx_U^2 %= U CONST
scoreboard players operation @s fx /= U CONST

scoreboard players operation @s fy *= @s dt
scoreboard players operation @s fy += @s err_vy_U^2
scoreboard players operation @s err_vy_U^2 = @s fy
scoreboard players operation @s err_vy_U^2 %= U CONST
scoreboard players operation @s fy /= U CONST

scoreboard players operation @s fz *= @s dt
scoreboard players operation @s fz += @s err_vz_U^2
scoreboard players operation @s err_vz_U^2 = @s fz
scoreboard players operation @s err_vz_U^2 %= U CONST
scoreboard players operation @s fz /= U CONST

# Multiply by radius (given in R)
scoreboard players operation @s fx *= @s radius
scoreboard players operation @s fx /= R CONST

scoreboard players operation @s fy *= @s radius
scoreboard players operation @s fy /= R CONST

scoreboard players operation @s fz *= @s radius
scoreboard players operation @s fz /= R CONST
